// Simulated game C functions
#include "manalink.h"

#include <stdio.h>
#include <setjmp.h>

static jmp_buf buf;

extern int cards[];

void lose_the_game(player_t player){
	if (!loseGameRecorder::instance().isRecording()) {
		// This is basically the original code (safe for speculating AIs and angels)
		real_lose_the_game(player);
	} else {
		// Mutex is set, append current call
		loseGameRecorder::instance().appendCall(player);
		longjmp(buf, 1); // to unravel the call stack
		// return; // <- DOESN'T WORK if someone other than draw_cards calls!!!
	}
}

// Class loseGameRecorder
// Implementation 
void loseGameRecorder::stopRecording() {
	if ( myMutex > 0)
		myMutex--;
	if ( myMutex == 0 ) {
		if (!myCalls.empty()) {
			lose_the_game(determineLooser());	
		}
	}
	return;
};
    
void loseGameRecorder::appendCall(player_t call) {
	try {
		myCalls.push_back(call);
	}
	catch (std::exception& e) {
		printf("Caught exception: %s\n", e.what());
		return;
	}
};

player_t loseGameRecorder::determineLooser() {
	try {
		player_t looser, nextCall;
		looser = myCalls.front();
		myCalls.pop_front();
		while (!myCalls.empty()) {
			nextCall = myCalls.front();
			myCalls.pop_front();
			if ((nextCall == ANYBODY) || (nextCall != looser)) return ANYBODY;
		}
		return looser;
	}
	catch (std::exception& e) {
		printf("Caught exception: %s\n", e.what());
		return ANYBODY;
	}
};

// drawing fuctions
int both_players_draw_naive(int amount) {
	
	draw_cards(HUMAN, amount);
	draw_cards(AI, amount);

	return 0;	
}

int both_players_draw_better(int amount) {
	
	loseGameRecorder::instance().startRecording();
    if (!setjmp(buf)) {
        draw_cards(HUMAN, amount);
    }
    if (!setjmp(buf)) {
        draw_cards(AI, amount);
    }
	loseGameRecorder::instance().stopRecording();

	return 0;	
}

// Laboratory Maniac
// In a perfect world this would be in innistrad.c, but I don't want to add anoher .c file
// It is sufficient that this is a separate function calling lose_the_game()
void laboratoryManiac(player_t player) {
	// We want to draw a card; if no cards are left win the game
	if (cards[player] == 0) {
		if (player == HUMAN)
			lose_the_game(AI);
		else
			lose_the_game(HUMAN);
	}
	
	return;
}

